package edu.ehu.GB.MIF.nasdaqGame;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import java.util.regex.*;

/**
 *
 * @author German
 */
public final class Nasdaq {
    private static final String[] tickers={"IBM","AAPL","MSFT","ORCL","HPQ","ADBE","INTC","GOOG","EBAY","YHOO"};
    private static final double comision=20;
    private static Statement st;
    
    //Posibilidad de especificar los meses a tomar de la web de Nasdaq
    //previa a la instanciación del singleton
    //Si no se hace, la intanciación usará el default
    private static int meses=-1;
    private static final int DEFAULT_MESES=2;
    public static void setMeses(int m){ if (meses==-1) meses=m;}
    public static int getMeses() {return meses;}
    
    //Singleton
    private static Nasdaq theInstance=null;
    public static Nasdaq getInstance() throws Exception{
        if (meses==-1) meses=DEFAULT_MESES;
        return theInstance!=null?theInstance:(theInstance=new Nasdaq(meses));
    }
    private Nasdaq(int meses) throws Exception {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
            java.sql.Connection con=java.sql.DriverManager.getConnection("jdbc:mysql://127.0.0.1:3306/","German", "chicle");
            st=con.createStatement();
            st.executeUpdate("use NasdaqPlay");
            st.executeUpdate("drop table if exists cotizaciones");
            st.executeUpdate("create table cotizaciones (ticker varchar(5),momento varchar(16) ,valor double)");
            Stock lastStock=null;
            for (String t:tickers) {
                for (Stock s:findStocks(t,meses)) {
                st.executeUpdate("insert into cotizaciones values('"+t+"','"+s.date+"_open' ,"+s.open+")");
                st.executeUpdate("insert into cotizaciones values('"+t+"','"+s.date+"_close' ,"+s.close+")");
                    lastStock=s;
                }
            }
            st.executeUpdate("insert into datos values ('"+lastStock.date+"_open')");
        
    }

    //Métodos de utilidad
    
    public static String[] getTickers() {
        return tickers;
    }
   
    public static double cotizacion(String ticker, String tradeInstant) {
        try {
            ResultSet rs=st.executeQuery("select valor from cotizaciones where ticker='"+ticker+"' and momento='"+tradeInstant+"'");
            rs.first();
            return rs.getDouble("valor");
        } catch(NullPointerException | SQLException ex) {return -1;}
    }
    
    public static String firstTradeInstant() {
         try {
            ResultSet rs=st.executeQuery("select ultimafecha from datos");
            rs.first();
            return rs.getString("ultimafecha");
        } catch(SQLException ex) {return null;}
    }
    
    public static double compra(String ticker, int cantidad, String tradeInstant) throws SQLException{
        return cantidad * cotizacion(ticker, tradeInstant) + comision;
    }

    public static double venta(String ticker, int cantidad, String tradeInstant) throws SQLException{
        return cantidad * cotizacion(ticker, tradeInstant) - comision;
    }
    
    
    //Copiadas de Mikel ----------- Debería estructurarse esto conforme a MVC
    private static class Stock {
	String date;
	double open,high,low,close;
	int vol;

	public Stock(String date,double open,double high,double low,double close,int volume) {
		this.date = date;
		this.open = open;
		this.high = high;
		this.low = low;
		this.close = close;
		this.vol = volume;
	}
	
	public String toString() {
		return "Date: " + date + " Open: " + open + " High: " + high + " Low: " + low + " Close: " + close + " Vol: " + vol;
	}

	public String toXML() {
		return 
				"<Stock" +
				" Date=\"" + date + "\"" + 
				" Open=\"" + open + "\"" + 
				" Max=\"" + high + "\"" + 
				" Min=\"" + low + "\"" + 
				" Close=\"" + close + "\"" + 
				" Vol=\"" + vol + "\"" + 
				"/>";
	}
	
}
    private static class WebPage {
	private int httpStatus;
	private String httpResponseMessage;
	private String content;
	
	public WebPage(int httpStatus, String httpResponseMessage, String content){
		this.httpStatus = httpStatus;
		this.httpResponseMessage = httpResponseMessage;
		this.content = content;
	}
	
	public String getContent(){
		return content;
	}
	
	public int getHttpStatus(){
		return httpStatus;
	}
	
	public String toString(){
		return "HTTP STATUS: " + httpStatus + " (" + httpResponseMessage + ")\n" + content;
	}
    }
    private static String getText(InputStream in) throws IOException {
		StringBuilder sb = new StringBuilder();
		try(
			BufferedReader r = new BufferedReader(new InputStreamReader(in));
		){
			String s;
			while ((s = r.readLine()) != null) sb.append(s).append('\n');
		}
		return sb.toString();
    }
    private static WebPage downloadWebPage(String address) throws MalformedURLException, IOException{
		URLConnection conn1 = new URL(address).openConnection();
		if (conn1 instanceof HttpURLConnection) {
			HttpURLConnection conn2 = (HttpURLConnection)conn1;
			int status = conn2.getResponseCode();
			return new WebPage(
				status,
				conn2.getResponseMessage(),
				getText(status < 400 ? conn2.getInputStream() : conn2.getErrorStream())
			);
		} else return null;
	}
    private static Stock[] findStocks(String ticker, int months) throws MalformedURLException, IOException {
		String url = "http://charting.nasdaq.com/ext/charts.dll?2-1-14-0-0-5"+ months +"-03NA000000" + ticker + "-&SF:6|5-WD=530-HT=395--XTBL-";
		WebPage wp = downloadWebPage(url);
		
		if (wp.getHttpStatus() < 400){
			List<Stock> stocks = new ArrayList<>();
			String regexp = "<TD CLASS=\"DrillDownDat[ae]\">(.*?)</TD>";
			Matcher m = Pattern.compile(regexp).matcher(wp.getContent());
			while (m.find()) {
				String data = m.group(1);
				double[] ohlc = new double[4];
				for (int i = 0; i < 4; i++) {
					m.find();
					ohlc[i] = Double.parseDouble(m.group(1));
				}
				m.find();
				int vol = Integer.parseInt(m.group(1).replaceAll(",", ""));
				Stock s = new Stock(data,ohlc[0],ohlc[1],ohlc[2],ohlc[3],vol);
				stocks.add(s);
			}
			return stocks.toArray(new Stock[0]);
		} else return null;
	}
 
}
